#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       05 May 2009
# Last Modified: $Date: 2009-06-09 15:11:32 +0100 (Tue, 09 Jun 2009) $
# Id:            $Id: histogram.pm 5570 2009-06-09 14:11:32Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/useful_modules/tags/release-10.0/lib/npg_common/diagram/histogram.pm $
#

package npg_common::diagram::histogram;

use strict;
use warnings;

use Readonly;
use Carp;
use English qw(-no_match_vars);

our $VERSION   = do { my ($r) = q$LastChangedRevision: 5570 $ =~ /(\d+)/mxs; $r; };

## no critic(Capitalization ProhibitMixedCaseVars ProhibitMagicNumbers ProhibitParensWithBuiltins RequireExtendedFormatting ProhibitInterpolationOfLiterals ProhibitCStyleForLoops ProhibitPostfixControls ProhibitUnlessBlocks  RequireLineBoundaryMatching RequireDotMatchAnything)

Readonly::Scalar our $DEFAULT_NUM_BINS   => 50;


sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;

    my $num_bins = shift;
    if ($num_bins && $num_bins > 0) {
	$self->{histogram_bins} = $num_bins;
    } else {
        $self->{histogram_bins} = $DEFAULT_NUM_BINS;
    }
    return $self;
}


sub get_num_bins {
    my $self = shift;
    return $self->{histogram_bins};
}


sub plot
{
    my $self = shift;
    my $dataArrRef = shift;

    #my $histogram_bins = $self->get('histogram_bins');
    my $histogram_bins = $self->{histogram_bins};

    my $cp = $self->_histogram_bins( $dataArrRef, $histogram_bins );

    my $binArrRef = $self->_histogram_frequency( $dataArrRef, $cp );

#    if ($self->get('histogram_type') eq 'percentage')
#    {
#    	    my $total = 0;
#	    grep($total += $_ , @$binArrRef);
#
#	    if ($total > 0)
#	    {
#	    	for (my $i = 0; $i < scalar(@$binArrRef); $i++)
#	    	{
#		    	$binArrRef->[$i] = 100 * $binArrRef->[$i] / $total;
#	    	}
#	    }
#    }
#
    my @labelArr;
    for my $bin (@{$cp})
    {
	    push(@labelArr, $self->_numformat( $bin->[0] + ($bin->[1] - $bin->[0])/2 ) );
    }

    my $results = [ \@labelArr, $binArrRef ];

    return $results;
}

###################################################################
# _histogram_bins - calculates the bins usings Scott's algorithm
#
# Arguements:
#
# $data  (Vector)  - Data values
#
# $nbins (Integer) - Number of bins to create. If $nbins is undef
#                    the number of bins is calculated using Scott's
#                    algorithm
#
###################################################################
sub _histogram_bins {
	my ( $self, $data, $nbins ) = @_;

	if( !defined $data ) { return; }

	my $calcBins = ( defined $nbins )? 0 : 1;
	my $cnt = 0;
	my $mean= 0;
	my $max = my $min = $data->[0];
	foreach (@{$data}) {
		$mean += $_;
		$min = ( $_ < $min )? $_ : $min;
		$max = ( $_ > $max )? $_ : $max;
		$cnt++;
	}
	$mean /= $cnt if( $cnt > 1 );

	my $sumsq = 0;
	$nbins = 1 if( $calcBins );
	my $s = 0;
	if( $cnt > 1 ) {
		foreach (@{$data}) {
			$sumsq += ( $_ - $mean )**2;
		}
		$s = sqrt( $sumsq / ($cnt - 1));
		$nbins = 3.49 * $s / $cnt**0.33 if( $s > 0 && $calcBins );
	}

	my $binwidth = ( $max - $min ) / $nbins;

	my $lower = $min;
	my $upper = $lower;

	my @cutPoints;
	my $cntr = 0;
	while ( $upper <= $max && $cntr < $nbins) {
		$upper = $lower + $binwidth;
		push @cutPoints, [$lower, $upper];
		$lower = $upper;
		$cntr++;
	}

	return \@cutPoints;
}

###################################################################
# _histogram_frequency - bins the data
#
#     Lower Boundry <= data value < Upper Boundry
#
# Arguements:
#
# $data  (Vector)  - Data values
#
# $nbins (Integer) - Vector containing the cutpoints to bin the data
#
###################################################################
sub _histogram_frequency {
	my ( $self, $data, $cutPoints ) = @_;

	if( !defined $data || !defined $cutPoints ) { return; }

	my @freqs;
	foreach (@{$cutPoints}) {
		push @freqs, 0;
	}

	foreach (@{$data})
	{
		for( my $i = 0; $i < scalar @{$cutPoints} ; $i++ )
		{
		if( ($_ >= $cutPoints->[$i]->[0] && $_ < $cutPoints->[$i]->[1])
			||
			($i == (scalar (@{$cutPoints}) - 1) && $_ >= $cutPoints->[$i]->[1]) )
			{

				$freqs[$i]++;
			}
		}
	}
	return \@freqs;
}

sub _numformat {
	my ($self, $v, $f1, $f2) = @_;

	unless(defined $v) { return $v; }

	unless(defined $f1) { $f1 = "%.4e"; }

	unless(defined $f2) {
		if ($v < 1) {
			$f2 = "%.5f";
		} else {
			$f2 = "%.3f";
		}
	}


    	## To display no for eg. 22.50 as 22.5
    	if ($v =~ /^([+-]?\d+)\.(\d+)$/) {
    		my $num = $1;
		my $fraction = $2;
		$fraction =~ s/0+$//;
		$v = (length($fraction) == 0) ? $num : "$num.$fraction";
	}

	if ($v =~ /\./){ ## no critic (RegularExpressions::ProhibitEscapedMetacharacters)
		if ($v == 0) {
			$v = 0;
		} elsif (($v > -0.001) and ($v < 0.001)) {
			$v = sprintf $f1, $v;
		} else {
			$v = sprintf $f2, $v;
		}
	}

	return $v;
}


1;
__END__

=head1 NAME

npg_common::diagram::histogram - Borrows code from GD::Graph::histogram, but does not need it

=head1 VERSION

$LastChangedRevision: 5570 $

=head1 SYNOPSIS

  use npg_common::diagram::histogram;

=head1 DESCRIPTION

npg_common::diagram::histogram creates a histogram from an array of data

=head1 USAGE

Fill an array with all the data values that are to be plotted. Note that
npg_common::diagram::histogram can only plot one data set at a time. 

	$data = [1,5,7,8,9,10,11,3,3,5,5,5,7,2,2];

Create the object

	my $h = new npg_common::diagram::histogram();

create the histogram

	my $histo = $h->plot($data);

=head1 OPTIONS

=head1 SUBROUTINES/METHODS

=head2 new creates a npg_common::diagram::histogram object
  # an object with a default number of bins (50)
  my $h =  npg_common::diagram::histogram->new();
  # an abject with a user-defined number of bins
  my $h =  npg_common::diagram::histogram->new(100);

=head2 plot creates a histogram; a ref to an array, returns a reference to an array of bins

=head2 get_num_bins  returnes currently set number of bins

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Readonly

=item GD::Graph::Data;

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

