#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: mg8 $
# Created:       03 April 2009
# Last Modified: $Date: 2009-04-08 18:30:40 +0100 (Wed, 08 Apr 2009) $
# Id:            $Id: fastq.pm 4830 2009-04-08 17:30:40Z mg8 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/trunk/lib/npg_common/extractor/fastq.pm $
#

package npg_common::extractor::fastq;

use strict;
use warnings;
use Carp;
use English qw(-no_match_vars);
use Math::Round qw(round);
use Exporter;
use Readonly;

our $VERSION   = do { my ($r) = q$LastChangedRevision: 5570 $ =~ /(\d+)/mxs; $r; };

## no critic (ProhibitExplicitISA)
our @ISA       = qw(Exporter);
our @EXPORT_OK = qw(line_count get_equally_spaced_reads);

Readonly our $LANES_PER_READ  => 4;


sub _jump_by {

    my ($fh1, $fh2, $count) = @_;
    my $i = 0;
    while ($i < $count ) {
        <$fh1> or return 0;
        <$fh2> or return 0;
        $i++;
    }

    return 1;
}


sub line_count {

  my $fname = shift;
  local ($ENV{PATH}) = $ENV{PATH} =~ /(.*)/smx;
  my $exe = qq[wc -l $fname];
  open my $fh, q[-|], $exe or croak $ERRNO;
  my $line_count = <$fh>;
  close $fh or croak $ERRNO;
  ($line_count) = $line_count =~ /(\d+)/smx;

  if (!$line_count) {
      croak "Failed to get a line count for file $fname";
  }

  return $line_count;
}


sub get_equally_spaced_reads {

    my ($fq1, $fq2, $fqe1, $fqe2, $num_reads) = @_;

    ## no critic (ProhibitMagicNumbers)
    if (@_ != 5) {
	croak q[get_short_fastq: Wrong number of arguments.];
    }
    ## use critic

    my $read_count = int line_count($fq1)/$LANES_PER_READ;
    if ($read_count < $num_reads) {
        croak qq[File $fq1 is too short - $read_count reads. At least $num_reads reads are required.];
    }

    my $scale_factor = $read_count / $num_reads;

    my $i = 0;
    my $read_index = 0;
    my $old_read_index = 0;
    my $jump_by = 0;
    my $line1 = q[];
    my $line2 = q[];
    my $end_loop = $LANES_PER_READ - 1;

    ## no critic (RequireBriefOpen)
    open my $source1, q[<], $fq1 or croak qq[Cannot open $fq1 for reading.];
    open my $source2, q[<], $fq2 or croak qq[Cannot open $fq2 for reading.];
    open my $dest1, q[>], $fqe1 or croak qq[Cannot open $fqe1 for writing.];
    open my $dest2, q[>], $fqe2 or croak qq[Cannot open $fqe2 for writing.];

    while ($i < $num_reads) {
        $old_read_index = $read_index;
        $read_index = round($i * $scale_factor);
        $jump_by = $read_index - $old_read_index - 1;
        if ($jump_by > 0) {
            _jump_by($source1, $source2, $jump_by * $LANES_PER_READ ) or last;
        }

        for my $j (0..$end_loop) {
           $line1 = <$source1>;
           $line2 = <$source2>;
           print {$dest1} $line1 or croak $ERRNO;
           print {$dest2} $line2 or croak $ERRNO;
	}

        $i += 1;
    }

    close $source1 or croak $ERRNO;
    close $source2 or croak $ERRNO;
    close $dest1 or croak $ERRNO;
    close $dest2 or croak $ERRNO;

    return 1;
}

1;

__END__

=head1 NAME

npg_common::extractor::fastq

=head1 VERSION

$LastChangedRevision: 5570 $

=head1 SYNOPSIS
  This module is for extracting parts of fastq files.

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 line_count - returns a number of lanes in a file
  my $count = line_count($my_path);

=head2 get_equally_spaced_reads - extracts reads from a fastq file; 
  the extracted reads are spaced equally through the file.
  get_equally_spaced_reads($fq1, $fq2, $fqe1, $fqe2, $num_reads), where
  $fq1 and $fq2 are paths to existing fastq files for a forward and reverse read;
  $fqe1 and $fqe2 are paths to fastq files for a forward and reverse read that
  have to be produced;
  $num_reads is the number of reads that should be in the output files.

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Readonly

=item Exporter

=item Math::Round qw(round);

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut


