#########
# Author:        $Author: ajb $
# Created:       2008-06-17
# Last Modified: $Date: 2009-06-09 15:11:32 +0100 (Tue, 09 Jun 2009) $
# Id:            $Id: Ref_Maker.pl 5570 2009-06-09 14:11:32Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/useful_modules/tags/release-10.0/scripts/Ref_Maker.pl $

package Ref_Maker;

use strict;
use warnings;
use Carp;
use Fatal qw( opendir mkdir );
use File::chdir;
use File::Copy;
use IPC::Open3;
use POSIX qw(WIFEXITED);
use Readonly;

our $VERSION = do { my ($r) = q$Revision: 5570 $ =~ /(\d+)/msx; $r; };

Readonly::Scalar my $DEBUG => 0;

Readonly::Scalar my $BWA_ALN_ALGORITHM_CUTOFF => 1_800_000_000;

Readonly::Scalar my $DATE_DAY_INDEX       =>    3;
Readonly::Scalar my $DATE_MONTH_INDEX     =>    4;
Readonly::Scalar my $DATE_YEAR_INDEX      =>    5;
Readonly::Scalar my $DATE_YEAR_CORRECTION => 1900;

Readonly::Scalar
     my $ALIGNER_REPOSITORY => '/software/solexa/bin/aligners/bin/';

my @aligners = qw( bowtie bwa maq samtools );
my ( %executable, %version_line, %command_line );


# First make sure we're in the right place.
croak 'No fasta directory found' if !-d 'fasta';
my $working_dir = $CWD;


# There should be one fasta file in the fasta subdirectory.
opendir FASTADIR, 'fasta';
my @files = grep {/[.]f(?:ast)?a$/imsx} readdir FASTADIR;
closedir FASTADIR;
croak 'One and only one fasta file expected' if scalar @files != 1;
my $reference = 'fasta/' . $files[0];


# bwa uses a different algorithm depending on the size of the reference.
# For files between 10MB and 2GB either can be used.
my $bwa_algorithm;
( -s $reference > $BWA_ALN_ALGORITHM_CUTOFF )
    ? ( $bwa_algorithm = 'bwtsw' )
    : ( $bwa_algorithm = 'is' );


foreach my $aligner (@aligners) {

    # Can we find and execute the aligner?
    $executable{$aligner} = $ALIGNER_REPOSITORY . $aligner;
    croak "$aligner not found/executable" if !-x $executable{$aligner};

    # Get the version number.
    my $command = $executable{$aligner};
    if ( $aligner eq 'bowtie' ) {
        $command .= ' --version';
    }

    open3( my $child_in, my $child_out, my $child_err, $command );
    while (<$child_out>) {
        next if $_ !~ m/version/imsx;
        $version_line{$aligner} = $_;
        last;
    }
    croak "Version string not found for $aligner" if !$version_line{$aligner};

    if ( $aligner eq 'bowtie' ) {
        $version_line{$aligner} =
            'Version ' . ( split /\s+/msx, $version_line{$aligner} )[2];
    }

    # Make the subdirectory.
    ( -d $aligner ) || ( mkdir $aligner );
}


# Do the conversions. Keep these out of a loop for now, as samtools and bwa
# are still in development and may change.

# samtools
$command_line{'samtools'} = "$executable{'samtools'} faidx $reference";
$DEBUG && print {*STDOUT} "\n$command_line{'samtools'}\n\n" || carp;
croak 'samtools conversion failed'
    if !WIFEXITED( system $command_line{'samtools'} );
croak 'fai not found' if !-f "$reference.fai";
move( "$reference.fai", "samtools/$files[0].fai" );


#bwa
$command_line{'bwa'} =
    "$executable{'bwa'} index -a $bwa_algorithm $reference";
$DEBUG && print {*STDOUT} "\n$command_line{'bwa'}\n\n" || carp;
croak 'bwa conversion failed' if !WIFEXITED( system $command_line{'bwa'} );
croak 'bwt not found' if !-f "$reference.bwt";
$CWD = 'fasta';
foreach my $index_file ( glob '*.*' ) {
    next if $index_file =~ m/[.]f(?:ast)?a$/imsx;
    move( $index_file, "../bwa/$index_file" );
}
$CWD = $working_dir;


#maq
$command_line{'maq'} =
    "$executable{'maq'} fasta2bfa $reference maq/$files[0].bfa";
$DEBUG && print {*STDOUT} "\n$command_line{'maq'}\n\n" || carp;
croak 'maq conversion failed' if !WIFEXITED( system $command_line{'maq'} );
croak 'maq not found' if !-f "maq/$files[0].bfa";


#bowtie
$CWD = 'bowtie';
$command_line{'bowtie'} =
    "$executable{'bowtie'}-build -f -q ../$reference $files[0]";
$DEBUG && print {*STDOUT} "\n$command_line{'bowtie'}\n\n" || carp;
croak 'bowtie conversion failed'
    if !WIFEXITED( system $command_line{'bowtie'} );
croak 'ebwt not found' if !-f "$files[0].1.ebwt";


# Print some text for the README file.
my ( $day, $month, $year ) =
    (localtime)[ $DATE_DAY_INDEX, $DATE_MONTH_INDEX, $DATE_YEAR_INDEX, ];
my $datestamp =
    sprintf '%d%02d%02d', $year + $DATE_YEAR_CORRECTION, $month++, $day;
print {*STDOUT} "\n\n\n$datestamp $ENV{'USER'}:\n" || carp;
foreach my $aligner (@aligners) {
    print {*STDOUT} "$aligner\t$version_line{$aligner}",
        "$command_line{$aligner}\n\n" || carp;
}

1;

__END__

=head1 NAME

    Ref_Maker

=head1 VERSION

    $Revision: 5570 $

=head1 SYNOPSIS

    # Change directory to location of fasta subdirectory in reference
    # repository.

    perl Ref_Maker.pl

=head1 DESCRIPTION

    Produce all the various reference files required for the various aligners
    used by NPD from a given fasta file.

    Call it from a given subset node of the reference repository, e.g. 'all'.
    It should be easy to write a wrapper to run it in all subset nodes below a
    given higher node - e.g. strain/version or organism.

    It finishes by printing some text suitable for pasting into the README
    file for the reference sequence, giving the version of the software used
    and the full command called to convert the reference.

=head1 DEPENDENCIES

=item Carp

=item Fatal

=item File::chdir

=item File::Copy

=item IPC::Open3

=item POSIX

=item Readonly

=item strict

=item warnings

=head1 BUGS AND LIMITATIONS

    There is no graceful error handling. Any problem at any stage causes the
    script to croak. It will say why though!

=head1 AUTHOR

    $Author: ajb $

=head1 LICENSE AND COPYRIGHT

=head1 SUBROUTINES/METHODS

    None.

=head1 DIAGNOSTICS

=head1 INCOMPATIBILITIES

=head1 CONFIGURATION AND ENVIRONMENT

    Requires a subdirectory called 'fasta' to be present in the working
    directory. This subdirectory should contain the fasta file to be converted
    and no other fasta files.

=cut

